classdef Observation < handle
    properties 
      time_obs
      mat_obs 
    end
    methods
function obs = Observation(time,mat)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Create an Observation object containing the data and the corresponding observation time
%
%USAGE:  obs = Observation(time,mat)
%INPUT :
%time                      :(vector or cell array object) contains the observation timestep under the form of a vector 
%                            or possibly a cell array of vectors if the observation times are different for each observed curve
%mat                       :(matrix of size length(time)x(number of observed curve) contains the observations under the form of a matrix
%                           or possibly a cell array of vectors if the observation times are different for each observed curve
%OUTPUT:
%obs                         :(Observation Object): return the constructed Observation object            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
if ismatrix(mat)
    col_d = length(mat(1,:));
    time_temp = time;
    T_cell =cell(col_d,1);
    M_cell = cell(col_d,1);
    for jh = 1 : col_d
        T_cell{jh} = time_temp;
        M_cell{jh} = mat(:,jh);
        
    end
    obs.time_obs =T_cell;
    obs.mat_obs = M_cell;
else
    obs.time_obs = time;
    obs.mat_obs = mat;
end

end

function Ti = getTimes(obs)
    Ti = rgetTimes(obs);
end

function setTimes(obs,time)
     obs.time_obs = time;
     %rsetTimes(obs,time)
end

function setMatrix(obs,matrix)
     obs.mat_obs = matrix;
end

function Mi = getMatrix(obs)
     Mi = obs.mat_obs;
end
    end 

end