function [var_param var_coeff_proj] = compute_variance_param_fitzHugh(bs,func_test,L,theta,Weight)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Variance estimation for both parameter and projection coefficient
%
%USAGE:   [var_param var_coeff_proj] = compute_variance_param_fitzHugh(bs,func_test,L,theta,Weight)
%
%INPUT :
%bs              :(Non-parametric curve estimator  object) , embedding the curve estimator
%func_test         :(function handler) used test function (try @splines_annulateur_borne)
%L               : (integer) number of test functions 
%theta     	     : (vector) first guess for parameter values
%(Optionnal)     :
%Weight          : weight matrix (default identity matrix)
%OUTPUT:
%var_param      :(matrix) result of the asymptotic variance-covariance matrix estimation (parameter)
%var_coeff_proj : (matrix) result of the asymptotic variance-covariance matrix estimation (projection coefficient)
%               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            obs =  bs.getObservation();
            T_c = obs.getTimes();
            T = T_c{1};
            %nb_t_calcul = length(T_calcul);
            T_0 = T(1);
            T_1 = T(length(T));

            if nargin < 5
                Weight =eye(L);
            end

    function res_aux = coeff_ftd_premiere(t)

         fonc_der_test = func_test(t,T_0,T_1,L,1,1)';        
         X_pred = bs.calcul_prediction(t,0);
         res_aux = zeros(L,1);

             for line =  1 : L            
            res_aux(line) = X_pred(1)*fonc_der_test(line);
             end
    end

     function res_aux = coeff_ftd_premiere_cube(t)

         fonc_der_test = func_test(t,T_0,T_1,L,1,1)';        
         X_pred = bs.calcul_prediction(t,0);
         res_aux = zeros(L,1);

             for line =  1 : L            
             res_aux(line) = (X_pred(1)-(X_pred(1)^3)/3)*fonc_der_test(line);
             end
    end

    function res_aux = coeff_ftd_zeros(t)

         fonc_dir_test = func_test(t,T_0,T_1,L,0,1)';        
         X_pred = bs.calcul_prediction(t,0);
         res_aux = zeros(L,1);

             for line =  1 : L            
                 res_aux(line,1) = X_pred(1)*fonc_dir_test(line);
             end
    end

    function res_aux = coeff_ftd_zeros_cube(t)

         fonc_dir_test = func_test(t,T_0,T_1,L,0,1)';        
         X_pred = bs.calcul_prediction(t,0);
         res_aux = zeros(L,1);

             for line =  1 : L            
                 res_aux(line,1) = ((X_pred(1).^3)/3)*fonc_dir_test(line);
             end
     end

    function res_aux = coeff_ftd(t)
        res_aux = func_test(t,T_0,T_1,L,0,1)';        
    end


    der_prem = quadv(@coeff_ftd_premiere,T_0,T_1);
    der_zeros = quadv(@coeff_ftd_zeros,T_0,T_1);
    der_prem_cube = quadv(@coeff_ftd_premiere_cube,T_0,T_1);
    der_zeros_cube = quadv(@coeff_ftd_zeros_cube,T_0,T_1);
    fonction_test = quadv(@coeff_ftd,T_0,T_1);
    
    Jacobien = zeros(L,3);
    Jacobien(:,1) = -fonction_test;
    Jacobien(:,2) = -(1/theta(3))*der_prem+ der_zeros_cube- der_zeros;
    Jacobien(:,3) = der_prem_cube+ theta(2)/(theta(3).^2)*der_prem;
  
    

function res =  compute_variance_coeff_fitzHugh(bs,fonc_test,L,theta)
 
    function res_aux = compute_aux(t)
        colonne_proj = bs.getValeurProjection(t,1);
        Volt = bs.calcul_prediction(t,0);
        
        fonc_dir_test = fonc_test(t,T_0,T_1,L,0,1)';
        fonc_der_test = fonc_test(t,T_0,T_1,L,1,1)';
        fonc_sec_test = fonc_test(t,T_0,T_1,L,2,1)';
        
        deux_partie = theta(3) - theta(3)*(Volt.^2) - theta(2)/theta(3);
        trois_partie = theta(2)*(Volt.^2) +(1-theta(2));
        
        
        ligne_l = ((fonc_sec_test)+(deux_partie*fonc_der_test)+(trois_partie*fonc_dir_test));

        
        res_aux = ligne_l*colonne_proj;
    end

 res_integ = quadv(@compute_aux,T_0,T_1);
 var_coeff = bs.getValeurVariance(1,2);
 res = res_integ*var_coeff*res_integ';
 
end
        M_mat = pinv(Jacobien'*Weight*Jacobien)*Jacobien'*Weight;
        var_coeff_proj =  compute_variance_coeff_fitzHugh(bs,func_test,L,theta);
        
%         size(var_coeff_proj)
%         size(M_mat)
        var_param = M_mat*var_coeff_proj*M_mat';

end