function [est_param est_variance interval_confiance] = minimize_variance_fitzHugh(bs,func_test,L,theta,alpha)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Parameter estimation by minimizing the variance
%
%USAGE:   [est_param est_variance interval_confiance] = minimize_variance_fitzHugh(bs,func_test,L,theta,alpha)
%
%INPUT :
%bs              :(Non-parametric curve estimator  object) , embedding the curve estimator
%func_test         :(function handler) used test function (try @splines_annulateur_borne)
%L               : (integer) number of test functions 
%theta     	     : (vector) first guess for parameter values
%(Optionnal)     :
% alpha          : (real) (1-confidence level) of the parameter confidence set (default 0.05)
%
%OUTPUT:
%est_param		    :(vector) result of the parameter estimation
%est_variance       :(matrix) result of the asymptotic variance-covariance matrix estimation 
%interval_confiance : confidence set for the parameter
%               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 5
    alpha = 0.05;
end

 p = norminv(1-alpha/2);
 param_estimator = calcul_param_fitzHugh(bs,theta,func_test,L);
 [var_param var_coeff_proj] = compute_variance_param_fitzHugh(bs,func_test,L,theta);
 
 poids_opt = pinv(var_coeff_proj);
 
 tol = 0.00001;
 ecart = tol +1;
 while ecart > tol
     
 nv_param_estimator = calcul_param_fitzHugh(bs,param_estimator,func_test,L,poids_opt);
 [var_param var_coeff_proj] = compute_variance_param_fitzHugh(bs,func_test,L,nv_param_estimator,poids_opt); 
 poids_opt = pinv(var_coeff_proj);
 ecart = sum(sum((param_estimator - nv_param_estimator).^2));
 param_estimator = nv_param_estimator    
 end
 
 est_param = param_estimator;
 est_variance = var_param;
 
 
  a_min = param_estimator(1) - p* sqrt(var_param(1,1));
  a_max = param_estimator(1) + p* sqrt(var_param(1,1));
  
  b_min = param_estimator(2) - p* sqrt(var_param(2,2));
  b_max = param_estimator(2) + p* sqrt(var_param(2,2));
  
  c_min = param_estimator(3) - p* sqrt(var_param(3,3));
  c_max = param_estimator(3) + p* sqrt(var_param(3,3));
  
  interval_confiance(1,:) = [a_min a_max];
  interval_confiance(2,:) = [b_min b_max];
  interval_confiance(3,:) = [c_min c_max];
  
end