function bs_u = est_control_pontryagin(bs,vect_field,der_vect_field,mat_Obs,mat_U,est_param,x_0,mat_b,param_fixe)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Compute optimal control for a given parameter value and initial condition
%
%USAGE:     bs_u = est_control_pontryagin(bs,vect_field,der_vect_field,mat_Obs,mat_U,est_param,x_0,param_fixe)
%
%INPUT :
%bs               :(bSplines  object) , embed the nonparametric estimators
%vect_field       :(function handler) embed the function corresponding to the vector field with signature: 
%                  res_t =  vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (vector of size dim x 1) vector field  value at time t
%der_vect_field       :(function handler) embed the function corresponding to the vector field derivative form x.rt state with signature: 
%                  res_t = der_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (matrix of size dim x dim) matrix der_f_i/x_j: vector field derivative form x.rt state value at time t
%mat_Obs                  : (matrix of size dim_obs x dim): observation matrix C
%mat_U                    : (matrix of size dim x dim): ponderation matrix U for control
%est_param                : (vector of size nb param) parameter value
%x_0                      :(vector ) initial condition 
%OPTIONAL
%param_fixe               :(vector) fixed parameter for the model (default empty ([]) )
%OUTPUT:
%bs_u                     : (bSplines object) estimated optimal control              
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

    Obs = bs.getObservation();
    T_c = Obs.getTimes();
    Times = T_c{1};
    invU = inv(mat_U);
    dim_syst = length(mat_Obs(1,:));
    dim_obs = length(mat_Obs(:,1));

    pas_solve =200;
    Int_ode_solve = Times(1):((Times(end)-Times(1))/pas_solve):Times(end); 
    
    if nargin < 9
        param_fixe = [];
    end
    
    if nargin < 8
        mat_b = eye(dim_syst);
    end
    dim_control =  length(mat_b(1,:));
    mat_weight_p = mat_b*invU *(mat_b');

    function res_t = champ_vect_state_param(t,vx,vp)
        res_t = vect_field(t,vx,est_param,param_fixe)+ mat_weight_p*vp/2;
    end


     function res_t = champ_vect_adjoint_param(t,vx,vp)
       y_hat = bs.calcul_prediction(t,0);
       y_hat = reshape(y_hat,dim_obs,1);

       res_t = 2*(mat_Obs')*mat_Obs*vx-(der_vect_field(t,vx,est_param,param_fixe)')*vp-2*(mat_Obs')*y_hat;
     end

     function dy = equ_b(t,y)
                val_x = reshape(y(1:dim_syst),dim_syst,1); 
                val_p = reshape(y((dim_syst+1):2*dim_syst),dim_syst,1); 

                dx = champ_vect_state_param(t,val_x,val_p);
                dp = champ_vect_adjoint_param(t,val_x,val_p);

                dx_v = reshape(dx,dim_syst,1);
                dp_v = reshape(dp,dim_syst,1);

                dy = [dx_v' dp_v']';

     end
 
         function dy = equ_b_ini(t,y)
                val_x = reshape(y,dim_syst,1);               
                dy = vect_field(t,val_x,est_param,param_fixe);
             end
         
         
           function sol_t_ini = func_sol_ini(t)
                sens_x_p = deval(sol_ini,t);
                sol_t_ini = zeros(2*dim_syst,1);
                sol_t_ini(1:dim_syst) = reshape(sens_x_p,dim_syst,1); 
           end
       
         function res = bcfun(sol_T_0,sol_T_end)
                res = zeros(2*dim_syst,1);
                res(1:dim_syst,1) = sol_T_0(1:dim_syst)-reshape(x_0,dim_syst,1);
                res((dim_syst+1):2*dim_syst,1) = sol_T_end((dim_syst+1):2*dim_syst);
         end
       
        sol_ini = ode45(@equ_b_ini,Int_ode_solve,reshape(x_0,dim_syst,1));      
        solinit = bvpinit(Int_ode_solve,@func_sol_ini);   
        options = bvpset('NMax',400);

        sol_equ =  bvp4c(@equ_b,@bcfun,solinit,options);
        
        
    function res_u_t = val_control_u_t(t)
           val_x_p = deval(sol_equ,t);

           val_p = reshape(val_x_p((dim_syst+1):2*dim_syst),dim_syst,1);
           res_u_t = invU*(mat_b')*val_p /2;
    end


    res_est_u_Times = arrayfun(@(t)(val_control_u_t(t)),Int_ode_solve','UniformOutput',false)';
    res_est_u_Times = cell2mat(res_est_u_Times)';
    bs_u = bSplines(Observation( Int_ode_solve,res_est_u_Times));
end