function [est_param,fval] = est_param_pontryagin_bvp4c_sp_version(bs,vect_field,der_vect_field,der_vect_field_theta,der_vect_field_state_theta,der_vect_field_state_state,mat_Obs,mat_U,param_ini,x_0,param_fixe)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Plot the economic function corresponding to the pontryagin estimator for non linear or linear model
%
%USAGE:   res_fval = plot_func_eco_pontryagin_general_case(bs,vect_field,der_vect_field,der_sec_vect_field,mat_Obs,mat_U,param_trial,x_0,param_fixe,type_solver)
%
%INPUT :
%bs               :(bSplines  object) , embed the nonparametric estimators
%vect_field       :(function handler) embed the function corresponding to the vector field with signature: 
%                  res_t =  vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (vector of size dim x 1) vector field  value at time t
%der_vect_field       :(function handler) embed the function corresponding to the vector field derivative form x.rt state with signature: 
%                  res_t = der_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (matrix of size dim x dim) matrix der_f_i/x_j: vector field derivative form x.rt state value at time t
%der_sec_vect_field       :(function handler) embed the function corresponding to the vector field second derivative form x.rt state with signature: 
%                  res_t = der_sec_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (hypermatrix of size dim x dim x dim) matrix der_sec_f_i/x_j_x_h: vector field second derivative form x.rt state value at time t
%mat_Obs                  : (matrix of size dim_obs x dim): observation matrix C
%mat_U                    : (matrix of size dim x dim): ponderation matrix U for control
%param_trial              : (matrix of size 1 x 2 or 2 x 2) lower and upper bound for varying parameter(s) value
%OPTIONAL
%x_0                      :(vector of size 1 x dim) initial condition if known (default empty ([]) )
%param_fixe               :(vector) fixed parameter for the model (default empty ([]) )
%type_solver              : (integer) indicate the used mixed boundary ode solver
%                         0 (default) extremal variation algorithm
%                         1           quasilinearisation algorithm
%
%OUTPUT:
%res_fval                 :(matrix or vector) economical function value for each varying parameter(s) value
%               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

    Obs = bs.getObservation();
    T_c = Obs.getTimes();
    Times = T_c{1};
    invU = inv(mat_U);
    dim_syst = length(mat_Obs(:,1));

    pas_solve =200;
    Int_ode_solve = Times(1):((Times(end)-Times(1))/pas_solve):Times(end);          
    
    free_ci = 0;
    if nargin < 10
        x_0 = [];
    end
    
     if isempty(x_0)
        free_ci = 1;
     end
    
    if nargin < 11
        param_fixe = [];
    end
    
     gradient_use = 1;
    if isempty(der_vect_field_theta) || isempty(der_vect_field_state_theta)  || isempty(der_vect_field_state_state)
        gradient_use = 0;
    end
    
    
    function res_t = champ_vect_state_param(t,vx,vp,param)
        res_t = vect_field(t,vx,param,param_fixe)+ invU*vp/2;
    end


     function res_t = champ_vect_adjoint_param(t,vx,vp,param)
       y_hat = bs.calcul_prediction(t,0);
       y_hat = reshape(y_hat,dim_syst,1);

       res_t = 2*(mat_Obs')*mat_Obs*vx-(der_vect_field(t,vx,param,param_fixe)')*vp-2*(mat_Obs')*y_hat;
     end
    

   


      function [res_val res_grad]= func_eco_multi_param(param)
          param

         nb_param_ci = length(param);
          
         if free_ci ==1
             nb_param = nb_param_ci - dim_syst;
             nv_x_0 = param((nb_param+1):end);
             param_tot = param(1:nb_param);
             nb_ci = dim_syst;
         else
             nb_param = nb_param_ci;
             nv_x_0 = x_0;
             param_tot = param;
             nb_ci = 0;
         end
          
         %nv_x_0
         nb_param = length(param_tot);
        
        
          function dy = equ_b(t,y)
                val_x = reshape(y(1:dim_syst),dim_syst,1); 
                val_p = reshape(y((dim_syst+1):2*dim_syst),dim_syst,1); 

                dx = champ_vect_state_param(t,val_x,val_p,param_tot);
                dp = champ_vect_adjoint_param(t,val_x,val_p,param_tot);

                dx_v = reshape(dx,dim_syst,1);
                dp_v = reshape(dp,dim_syst,1);

                dy = [dx_v' dp_v']';

          end
       
            function res = bcfun(sol_T_0,sol_T_end)
                res = zeros(2*dim_syst,1);
                res(1:dim_syst,1) = sol_T_0(1:dim_syst)-reshape(nv_x_0,dim_syst,1);
                res((dim_syst+1):2*dim_syst,1) = sol_T_end((dim_syst+1):2*dim_syst);
            end
            
            function dy = equ_b_ini(t,y)
                val_x = reshape(y,dim_syst,1);               
                dy = vect_field(t,val_x,param_tot,param_fixe);
             end
         
         
           function sol_t_ini = func_sol_ini(t)
                sens_x_p = deval(sol_ini,t);
                sol_t_ini = zeros(2*dim_syst,1);
                sol_t_ini(1:dim_syst) = reshape(sens_x_p,dim_syst,1); 
           end

         

        if gradient_use ==0   
         sol_ini = ode45(@equ_b_ini,Int_ode_solve,reshape(nv_x_0,dim_syst,1));      
         solinit = bvpinit(Int_ode_solve,@func_sol_ini);    
         sol_equ =  bvp4c(@equ_b,@bcfun,solinit);
        end
        
        ind_fin_p = 2*dim_syst;
        ind_fin_x_sens_par = ind_fin_p+dim_syst*nb_param;
        ind_fin_p_sens_par = ind_fin_x_sens_par + dim_syst*nb_param;
        ind_fin_x_sens_x0 = ind_fin_p_sens_par +dim_syst*dim_syst;
        ind_fin_x_sens_p = ind_fin_x_sens_x0 +dim_syst*dim_syst;

         function dy = equ_sens_tot_b(t,y)
                val_x = reshape(y(1:dim_syst),dim_syst,1); 
                val_p = reshape(y((dim_syst+1):2*dim_syst),dim_syst,1); 

                val_x_sens_par = y((ind_fin_p+1):ind_fin_x_sens_par);
                
                val_p_sens_par = y((ind_fin_x_sens_par+1):ind_fin_p_sens_par);
                
                val_x_sens = reshape(val_x_sens_par,dim_syst,nb_param);
                val_p_sens_par = reshape(val_p_sens_par,dim_syst,nb_param);
                
                dx = champ_vect_state_param(t,val_x,val_p,param_tot);
                dp = champ_vect_adjoint_param(t,val_x,val_p,param_tot);

                dx_v = reshape(dx,dim_syst,1);
                dp_v = reshape(dp,dim_syst,1);

                
                df_state_theta = der_vect_field_state_theta(t,val_x,param_tot,param_fixe);
                df_state_state = der_vect_field_state_state(t,val_x,param_tot,param_fixe);
                df_theta  = der_vect_field_theta(t,val_x,param_tot,param_fixe);
                df_state  = der_vect_field(t,val_x,param_tot,param_fixe);
                
                
                part_sens_p_der_theta_inter = zeros(dim_syst,nb_param);
                part_sens_p_der_x_inter = zeros(dim_syst);
                
                for nd = 1 : dim_syst
                     part_sens_p_der_theta_inter =  part_sens_p_der_theta_inter +df_state_theta(:,:,nd)*val_p(nd);
                     part_sens_p_der_x_inter = part_sens_p_der_x_inter + df_state_state(:,:,nd)*val_p(nd);
                end
                
                part_sens_p_der_theta = -part_sens_p_der_theta_inter;
                part_sens_p_der_x = - part_sens_p_der_x_inter +2*(mat_Obs')*mat_Obs;
                
                d_val_x_sens_par = df_theta+(invU/2)*val_p_sens_par+df_state*val_x_sens_par;
                d_val_p_sens_par = part_sens_p_der_theta-(df_state')*val_p_sens_par + part_sens_p_der_x*val_x_sens_par;
                
                d_val_x_sens_par_v = reshape(d_val_x_sens_par,1,dim_syst*nb_param);
                d_val_p_sens_par_v = reshape(d_val_p_sens_par,1,dim_syst*nb_param);
                
                dy = [dx_v dp_v d_val_x_sens_par_v d_val_p_sens_par_v]';
                
                
                if nb_ci > 0
                    val_x_sens_x0 = y((ind_fin_p_sens_par+1):ind_fin_x_sens_x0);
                    val_p_sens_x0 = y((ind_fin_x_sens_x0+1):ind_fin_x_sens_p);

                    val_x_sens_x0 = reshape(val_x_sens_x0,dim_syst,dim_syst);
                    val_p_sens_x0 = reshape(val_p_sens_x0,dim_syst,dim_syst);

                    part_sens_p_der_x_inter = zeros(dim_syst,dim_syst);

                    for nd = 1 : dim_syst
                         part_sens_p_der_x_inter = part_sens_p_der_x_inter + df_state_state(:,:,nd)*val_p(nd);
                    end

                    part_sens_p_der_x = - part_sens_p_der_x_inter +2*(mat_Obs')*mat_Obs;

                    d_val_x_sens_x0 = (invU/2)*val_p_sens_x0+df_state*val_x_sens_x0;
                    d_val_p_sens_x0 = -(df_state')*val_p_sens_x0 + part_sens_p_der_x*val_x_sens_x0;
                    
                    d_val_x_sens_x0_v = reshape(d_val_x_sens_x0,1,dim_syst*dim_syst);
                    d_val_p_sens_x0_v = reshape(d_val_p_sens_x0,1,dim_syst*dim_syst);

                    dy = [dy' d_val_x_sens_x0_v d_val_p_sens_x0_v]';
                end
         end
          
         
         function res = bcfun_sens_tot(sol_sens_tot_T_0,sol_sens_tot_T_end)
             
             if free_ci == 1
                res = zeros(2*(dim_syst*nb_param +dim_syst+dim_syst*dim_syst),1);
             else
                 res = zeros(2*(dim_syst*nb_param +dim_syst),1);
             end
             
                res(1:dim_syst,1) = sol_sens_tot_T_0(1:dim_syst)-reshape(nv_x_0,dim_syst,1);
                res((dim_syst+1):ind_fin_p,1) = sol_sens_tot_T_end((dim_syst+1):ind_fin_p);
                res((ind_fin_p+1):ind_fin_x_sens_par,1) = sol_sens_tot_T_0((ind_fin_p+1):ind_fin_x_sens_par);
                res((ind_fin_x_sens_par+1):ind_fin_p_sens_par,1) = sol_sens_tot_T_end((ind_fin_x_sens_par+1):ind_fin_p_sens_par);
                
                 if free_ci == 1
                    res((ind_fin_p_sens_par+1):ind_fin_x_sens_x0,1) = sol_sens_tot_T_end((ind_fin_p_sens_par+1):ind_fin_x_sens_x0) - reshape(eye(dim_syst,dim_syst),dim_syst*dim_syst,1);
                    res((ind_fin_x_sens_x0+1):ind_fin_x_sens_p,1) = sol_sens_tot_T_end((ind_fin_x_sens_x0+1):ind_fin_x_sens_p);
                 end
         end
         
         
         function dy = equ_sens_b_ini(t,y)
                val_x = y(1:dim_syst);
                val_x = reshape(val_x,dim_syst,1);
                val_x_sens_p = y((dim_syst+1):(dim_syst+dim_syst*nb_param));
                val_x_sens_p = reshape(val_x_sens_p,dim_syst,nb_param);
                

                df_theta  = der_vect_field_theta(t,val_x,param_tot,param_fixe);
                df_state  = der_vect_field(t,val_x,param_tot,param_fixe);
                
                d_val_x = vect_field(t,val_x,param_tot,param_fixe);
                d_val_x_sens_p = df_theta+df_state*val_x_sens_p;
                
                d_val_x_v = reshape(d_val_x,1,dim_syst);
                d_val_x_sens_p_v = reshape(d_val_x_sens_p,1,dim_syst*nb_param);
                
                dy = [d_val_x_v d_val_x_sens_p_v]';
                
                
                if free_ci == 1
                  val_x_sens_x0 = y((dim_syst+dim_syst*nb_param+1):(dim_syst+dim_syst*nb_param+dim_syst*dim_syst));
                  val_x_sens_x0 = reshape(val_x_sens_x0,dim_syst,dim_syst);
                  d_val_x_sens_x0 = df_state*val_x_sens_x0;
                  d_val_x_sens_x0_v = reshape(d_val_x_sens_x0,1,dim_syst*dim_syst);
                  dy = [dy' d_val_x_sens_x0_v]';
                end
         end
         
         
           function sol_t_ini = func_sol_ini_sens(t)
               
                      
             if free_ci == 1
                 sol_t_ini = zeros(2*(dim_syst*nb_param +dim_syst+dim_syst*dim_syst),1);
             else
                 sol_t_ini = zeros(2*(dim_syst*nb_param +dim_syst),1);
             end
             
                sens_x_p_nul = deval(sol_equ_sens_ini,t);
                val_x_p_nul = sens_x_p_nul(1:dim_syst);
                val_x_sens_p_nul = sens_x_p_nul((dim_syst+1):(dim_syst+dim_syst*nb_param));
                
                sol_t_ini(1:dim_syst) = reshape(val_x_p_nul,dim_syst,1); 
                sol_t_ini((ind_fin_p+1):ind_fin_x_sens_par) = reshape(val_x_sens_p_nul,dim_syst*nb_param,1); 
                
                
                if free_ci == 1
                     val_x_sens_x0_p_nul = sens_x_p_nul((dim_syst+dim_syst*nb_param+1):(dim_syst+dim_syst*nb_param+dim_syst*dim_syst));
                     sol_t_ini((ind_fin_p_sens_par+1):ind_fin_x_sens_x0) = reshape(val_x_sens_x0_p_nul,dim_syst*dim_syst,1); 
                end
           end

         
         if gradient_use ==1
             if free_ci == 1
              CI_ini = [reshape(nv_x_0,1,dim_syst) zeros(1,nb_param*dim_syst) reshape(eye(dim_syst),1,dim_syst*dim_syst)] ;   
             else
              CI_ini = [reshape(nv_x_0,1,dim_syst) zeros(1,nb_param*dim_syst)] ;
             end
             sol_equ_sens_ini = ode45(@equ_sens_b_ini,Int_ode_solve,CI_ini);   
             solinit_sens = bvpinit(Int_ode_solve,@func_sol_ini_sens);    
             sol_equ_sens =  bvp4c(@equ_sens_tot_b,@bcfun_sens_tot,solinit_sens);
         end
         
 
         
         function res_t = func_inner_integ(t)
             
                res_t = zeros(1+(nb_param+nb_ci)*gradient_use,1);
                y_hat = bs.calcul_prediction(t,0);
                y_hat = reshape(y_hat,dim_syst,1);
            
          if gradient_use == 1   
            val_x_p_sens_tot = deval(sol_equ_sens,t);
            
                val_x = reshape(val_x_p_sens_tot(1:dim_syst),dim_syst,1); 
                val_p = reshape(val_x_p_sens_tot((dim_syst+1):ind_fin_p),dim_syst,1); 

                val_x_sens_p = val_x_p_sens_tot((ind_fin_p+1):ind_fin_x_sens_par);
                
                val_p_sens_p = val_x_p_sens_tot((ind_fin_x_sens_par+1):ind_fin_p_sens_par);
                
                val_x_sens_p = reshape(val_x_sens_p,dim_syst,nb_param);
                val_p_sens_p = reshape(val_p_sens_p,dim_syst,nb_param);
                
            
            val_u = invU*val_p/2;
            
            res_t(1) = sum((mat_Obs*val_x - y_hat).^2) + val_u'*mat_U*val_u;
            
                val_x_sens_tot = zeros(dim_syst,nb_param+nb_ci);
                val_p_sens_tot = zeros(dim_syst,nb_param+nb_ci);


                val_x_sens_tot(:,1:nb_param) = val_x_sens_p;
                val_p_sens_tot(:,1:nb_param) = val_p_sens_p;

                if nb_ci > 0
                    val_x_sens_xT =  val_x_p_sens_tot((ind_fin_p_sens_par+1):ind_fin_x_sens_x0);
                    val_p_sens_xT =  val_x_p_sens_tot((ind_fin_x_sens_x0+1):ind_fin_x_sens_p);
                    val_x_sens_xT = reshape(val_x_sens_xT,dim_syst,dim_syst);
                    val_p_sens_xT = reshape(val_p_sens_xT,dim_syst,dim_syst);

                    val_x_sens_tot(:,(nb_param+1):(nb_param+dim_syst)) = val_x_sens_xT;
                    val_p_sens_tot(:,(nb_param+1):(nb_param+dim_syst)) = val_p_sens_xT;
                end


                res_t(2:end) = 2*(val_x_sens_tot')*(mat_Obs')*(mat_Obs*val_x - y_hat) + (val_p_sens_tot')*invU*val_p/2;
          else

                val_x_p = deval(sol_equ,t);

                val_x = reshape(val_x_p(1:dim_syst),dim_syst,1);
                val_p = reshape(val_x_p((dim_syst+1):2*dim_syst),dim_syst,1);

                val_u = invU*val_p/2;

                res_t(1) = sum((mat_Obs*val_x - y_hat).^2) + val_u'*mat_U*val_u;
          end
        end
        
        res_integration = quadv(@func_inner_integ,Times(1),Times(end));      
        res_val = res_integration(1);
        
        if gradient_use ==1
           res_grad = res_integration(2:end);
        end
        
      end
  
  
       if gradient_use == 1
        nb_fun_eval = 100000;
        max_iter = 1000;
        options = optimset('TolFun',1e-10,'TolX',1e-10,'GradObj','on','MaxFunEvals',nb_fun_eval,'MaxIter',max_iter,'Hessian','off');
        [est_param,fval]= fminunc(@func_eco_multi_param,param_ini,options);
      else
       [est_param,fval]= fminsearch(@func_eco_multi_param,param_ini);
      end


end