function [est_param_opt_non_param,fval_opt_non_param,lambda_opt_non_param,est_param_opt_data,fval_opt_data,lambda_opt_data, mat_est, mat_fval,est_param_opt_non_param_cor,fval_opt_non_param_cor,lambda_opt_non_param_cor,est_param_opt_data_cor,fval_opt_data_cor,lambda_opt_data_cor]...
    = est_param_pontryagin_lambda_trial(bs,vect_field,der_vect_field,der_vect_field_theta,der_vect_field_state_theta,der_vect_field_state_state,mat_Obs,lambda_trial,param_ini,x_0,mat_b,type_solver,bvp_solv,param_fixe)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Compute the Pontryagin Maximum estimator for a given value of lambda
%
%USAGE:    [est_param_opt_non_param,fval_opt_non_param,lambda_opt_non_param,est_param_opt_data,fval_opt_data,lambda_opt_data, mat_est, mat_fval,est_param_opt_non_param_cor,fval_opt_non_param_cor,lambda_opt_non_param_cor,est_param_opt_data_cor,fval_opt_data_cor,lambda_opt_data_cor]...
%    = est_param_pontryagin_lambda_trial(bs,vect_field,der_vect_field,der_vect_field_theta,der_vect_field_state_theta,der_vect_field_state_state,mat_Obs,lambda_trial,param_ini,x_0_known,type_solver,bvp_solv,param_fixe)
%INPUT :
%bs               :(bSplines  object) , embed the nonparametric estimators
%vect_field       :(function handler) embed the function corresponding to the vector field with signature: 
%                  res_t =  vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (vector of size dim x 1) vector field  value at time t
%der_vect_field       :(function handler) embed the function corresponding to the vector field derivative form x.rt state with signature: 
%                  res_t = der_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (matrix of size dim x dim) matrix der_f_i/x_j: vector field derivative form x.rt state value at time t
%der_vect_field_theta       :(function handler) embed the function corresponding to the vector field derivative form x.rt parameter with signature: 
%                  res_t = der_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (matrix of size dim x nb param) matrix der_f_i/theta_j: vector field derivative form x.rt parameter value at time t
%der_vect_field_state_theta       :(function handler) embed the function corresponding to the vector field second derivative form x.rt state and parameter with signature: 
%                  res_t = der_sec_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (hypermatrix of size dim x nb param x dim) matrix der_sec_f_i/x_j_theta_h: vector field second derivative form x.rt state and parameter value at time t
%der_vect_field_state_state       :(function handler) embed the function corresponding to the vector field second derivative form x.rt state with signature: 
%                  res_t = der_sec_vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (hypermatrix of size dim x dim x dim) matrix der_sec_f_i/x_j_x_h: vector field second derivative form x.rt state value at time t
%mat_Obs                  : (matrix of size dim_obs x dim): observation matrix C
%lambda_trial             : (vector): trial of tested smoothing parameter value
%param_ini                : (matrix of size nb param) initial guess for the optimization algorithm
%OPTIONAL
%x_0_known                :(vector ) initial condition if known (default empty [] )
%bvp_solv                 : (integer) indicate the used mixed boundary ode solver
%                         1 (default) bvp4c
%                         0           bvp5c
%type_solv                :(integer) indicate the used optimization matlab function
%                          1 (default) fminunc
%                          0 fmincon
%param_fixe               :(vector) fixed parameter for the model (default empty ([]) )
%OUTPUT:
%est_param_opt_non_param                   :(p sized vector): selected estimation minimizing L2 discrepancy between true model and X_hat
%fval_opt_non_param                        :(real) corresponding minimum cost value 
%lambda_opt_non_param                      :(real) corresponding lambda value 
%est_param_opt_data                        :(p sized vector): selected estimation minimizing  discrepancy between true model and observations
%fval_opt_data                             :(real) corresponding minimum cost value 
%lambda_opt_data                           :(real) corresponding lambda value 
%mat_est                                   :(matrix) parameter estimation set obtained for the tested lambda values
%mat_fval                                  :(vector) obtained minimum cost values for the tested lambda values
%est_param_opt_non_param_cor               :(p sized vector): selected estimation minimizing L2 discrepancy between corrected model and X_hat
%fval_opt_non_param_cor                    :(real) corresponding minimum cost value 
%lambda_opt_non_param_cor                  :(real) corresponding lambda value 
%est_param_opt_data_cor                    :(p sized vector): selected estimation minimizing  discrepancy  between corrected model and observations
%fval_opt_data_cor                         :(real) corresponding minimum cost value 
%lambda_opt_data_cor                       :(real) corresponding lambda value          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
 
  
    dim_syst = length(mat_Obs(1,:));
    nb_param = length(param_ini);
         
    warning off
    free_ci = 0;
    if nargin < 10
        x_0 = [];
    end
    
     if isempty(x_0)
        free_ci = 1;
     end
    
    if nargin < 14
        param_fixe = [];
    end
    
    if nargin < 13
        bvp_solv = 1;
    end
    
    if nargin < 12
        type_solver = 1;
    end
    
    if nargin < 11
        mat_b = eye(dim_syst);
    end
    dim_control =  length(mat_b(1,:));
    
    mat_est = zeros(length(lambda_trial),nb_param);
    mat_fval = zeros(length(lambda_trial),1);

    mat_dec_non_param = zeros(length(lambda_trial),1);
    mat_dec_data = zeros(length(lambda_trial),1);
    mat_dec_non_param_cor = zeros(length(lambda_trial),1);
    mat_dec_data_cor = zeros(length(lambda_trial),1);


    for nl = 1 : length(lambda_trial)
        
        if isa(lambda_trial,'cell')
            Mat_nl = lambda_trial{nl};
        else
            Mat_nl = eye(dim_control)*lambda_trial(nl);
            lambda_trial(nl)
        end
        
        param_ini_algo = param_ini;
        if nl > 1
           est_cur  = mat_est(nl-1,:)
           val_sum_abs_est_cur = sum(abs(est_cur))
            if val_sum_abs_est_cur > 0.2
                param_ini_algo = est_cur;
            end
        end
        
        
       try
       %Mat_nl           
       [est_param,fval,bs_u] = est_param_pontryagin_bvp4c_adjoint_method(bs,vect_field,der_vect_field,der_vect_field_theta,der_vect_field_state_theta,der_vect_field_state_state,mat_Obs,Mat_nl,param_ini_algo,x_0,mat_b,type_solver,bvp_solv,param_fixe);

       %est_param
          if free_ci == 1
                est_param_theta = est_param(1:(length(est_param)-dim_syst));
                est_param_x_0 = est_param((length(est_param)-dim_syst+1):end); 
             else
                est_param_theta = est_param;
                est_param_x_0 = x_0; 
          end
            
       [dist_est_non_param dist_est_data] = est_pontryagin_calcul_distance(bs,vect_field,mat_Obs,est_param_theta,est_param_x_0,[],mat_b,param_fixe);
       [dist_est_non_param_cor dist_est_data_cor] = est_pontryagin_calcul_distance(bs,vect_field,mat_Obs,est_param_theta,est_param_x_0,@(t)(bs_u.calcul_prediction(t,0)),mat_b,param_fixe);
        
        mat_est(nl,:) = est_param;
        
        mat_fval(nl) = fval;
        mat_dec_non_param(nl) =dist_est_non_param;
        mat_dec_data(nl) =dist_est_data;
        mat_dec_non_param_cor(nl) = dist_est_non_param_cor;
        mat_dec_data_cor(nl) = dist_est_data_cor;
        
       catch
              
            mat_est(nl,:) = zeros(1,nb_param);

            mat_fval(nl) = 10^20;
            mat_dec_non_param(nl) =10^20;
            mat_dec_data(nl) =10^20;
            mat_dec_non_param_cor(nl) =10^20;
            mat_dec_data_cor(nl) = 10^20;
      end

    end

    [val_min, ind_min_non_param] = min(mat_dec_non_param);
     est_param_opt_non_param = mat_est(ind_min_non_param,:);
     lambda_opt_non_param = lambda_trial(ind_min_non_param);
     fval_opt_non_param= mat_fval(ind_min_non_param);
     
      [val_min, ind_min_data] = min(mat_dec_data);
     est_param_opt_data = mat_est(ind_min_data,:);
     lambda_opt_data = lambda_trial(ind_min_data);
     fval_opt_data= mat_fval(ind_min_data);
     
     [val_min_cor, ind_min_non_param_cor] = min(mat_dec_non_param_cor);
      est_param_opt_non_param_cor = mat_est(ind_min_non_param_cor,:);
     lambda_opt_non_param_cor = lambda_trial(ind_min_non_param_cor);
     fval_opt_non_param_cor = mat_fval(ind_min_non_param_cor);

     [val_min_cor, ind_min_data_cor] = min(mat_dec_data_cor);
     est_param_opt_data_cor = mat_est(ind_min_data_cor,:);
     lambda_opt_data_cor = lambda_trial(ind_min_data_cor);
     fval_opt_data_cor = mat_fval(ind_min_data_cor);
     warning on
end