function res_err_pred = pontryagin_prediction_error_semiparam_multi_inter(bs_l,bs_u,v_field_ext,v_field,mat_C,est_param,x0_ext,intervalle_int,mat_B_ext)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Compute the quantifier of the prediction error used for lambda selection
%
%USAGE:    res_err_pred = pontryagin_prediction_error_semiparam_multi_inter(bs_l,bs_u,v_field_ext,v_field,mat_C,est_param,x0_ext,intervalle_int,mat_B_ext)
%
%INPUT :
%bs               :(bSplines  object) , embed the nonparametric estimator
%bs               :(bSplines  object) , embed the estimated control
%vect_field       :(function handler) embed the function corresponding to the vector field used to compute the forced solution X_u with signature: 
%                  res_t =  vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (vector of size dim x 1) vector field  value at time t
%vect_field       :(function handler) embed the function corresponding to the initial vector field with signature: 
%                  res_t =  vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (vector of size dim x 1) vector field  value at time t
%mat_C                    : (matrix of size dim_obs x dim): observation matrix C
%est_param                : (vector) estimated value of the parameters
%x0_ext                   : (vector) estimated or true value of the initial conditions
%intervalle_int           : (matrix) matrix with  2 columns containing on each row one subinterval
%mat_B_ext                : (matrix) matrice B in the forced model.
%OUTPUT
%res_err_pred             : predicton error         
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    Obs = bs_l.getObservation();
    T_c = Obs.getTimes();
    Times = T_c{1};
    Data_c = Obs.getMatrix();
    res_data = cell2mat(Data_c');
    dim_syst_ext = length(x0_ext);
    dim_syst = length(mat_C(1,:));
    pas_solve =400;
    Int_ode_solve = Times(1):((Times(end)-Times(1))/pas_solve):Times(end); 
    
    nb_obs =  length(Times);
    nb_inter_tot = length(intervalle_int(:,1));

    function res_t = champ_vect_controlled(t,vx)
        u_t = bs_u.calcul_prediction(t,0);
        res_t = v_field_ext(t,vx,est_param,[])+ mat_B_ext*reshape(u_t,length(u_t),1);
    end

    function res_t = champ_vect_uncontrolled(t,vx)
        res_t = v_field(t,vx,est_param);
    end

    sol_controlled = ode45(@champ_vect_controlled,Int_ode_solve,reshape(x0_ext,dim_syst_ext,1)); 
    
    sum_err_pred = 0;

              for ninter = 1 : nb_inter_tot
                 Inter_n_int = intervalle_int(ninter,:);
                 Times_n_int = Times(find(Times==Inter_n_int(1)):find(Times==Inter_n_int(2)));
                 nb_obs_ninter = length(Times_n_int);
                 
                 Int_ode_solve_cur = Inter_n_int(1):((Inter_n_int(2)-Inter_n_int(1))/pas_solve):Inter_n_int(2);
                 val_ini_ninter = deval(sol_controlled,Times_n_int(1));
                 sol_uncontrolled_ninter = ode45(@champ_vect_uncontrolled,Int_ode_solve_cur,reshape(val_ini_ninter(1:dim_syst),dim_syst,1));
                  
                 for i = 1 : nb_obs_ninter
                     val_pred_i = deval(sol_uncontrolled_ninter,Times_n_int(i));
                     indice_i = find(Times==Times_n_int(i));
                     
                     sum_err_pred = sum_err_pred  + sum((res_data(indice_i,:)' - mat_C*val_pred_i).^2);
                 end
                  
              end
                 res_err_pred = sum_err_pred/nb_obs;
                            
end