% First, in order to set up Monte Carlo data, we need a function to put
% into the ode solver.

odefn    = @fhnfunode;      % Function for ODE solver (exact)

% First we need a function to calculate the functionl directly.

fn.fn       = @fhnfun;       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

fn.dfdx     = @fhndfdx;      % Derivative wrt inputs (Jacobian)
fn.dfdp     = @fhndfdp;      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

fn.d2fdx2   = @fhnd2fdx2;    % Hessian wrt inputs
fn.d2fdxdp  = @fhnd2fdxdp;   % Hessian wrt inputs and parameters
fn.d2fdp2   = @fhnd2fdp2;    % Hessian wrt parameters.


deb=0;
fin=20;
h_var = 0.1;

x0= [-1 1];
a = 0.2;
b = 0.2;
c = 2+1;
st_var = 0.2;

theta = [a b c];

  [T1 Y1] = fitz_nagumo(deb,fin,h_var,x0,a,b,c,st_var);
  
  Tcell = cell(1,2);
  Tcell{1} = T1;
  Tcell{2} = T1;
  
  Ycell = cell(1,2);
  Ycell{1} = Y1(:,1);
  Ycell{2} = Y1(:,2);

wts = [];


% Alternatively, we may decide to weight each component according to its
% scale. This can be done by associating a weight given by the inverse of
% the variance of the observations in that component.

for i = 1:length(Ycell)
    if ~isempty(Ycell{i})
        wts(i) = 1./sqrt(var(path_cell{i}));
    else
        wts(i) = 1;
    end
end


lambda = 1e4;
lambda = lambda * wts;
lambda0 = 1;



nknots = 401;
norder = 3;
nquad = 5;
lsopts_out = optimset('DerivativeCheck','off','Jacobian','on',...
    'Display','iter','MaxIter',1000,'TolFun',1e-8,'TolX',1e-10);

lsopts_in = optimset('DerivativeCheck','off','Jacobian','on',...
    'Display','off','MaxIter',1000,'TolFun',1e-14,'TolX',1e-14,...
    'JacobMult',@SparseJMfun);

range = [min(full_time),max(full_time)];
knots = linspace(range(1),range(2),nknots);
nbasis = nknots+norder-2;
quadvals = [knots' ones(length(knots),1)/nknots];
basis_cell = cell(1,length(Ycell));


Lfd_cell = cell(1,length(Ycell));

bbasis = MakeBasis(range,nbasis,norder,knots,quadvals,1);

basis_cell(:) = {bbasis};
Lfd_cell(:) = {fdPar(bbasis,1,lambda0)};

DEfd = smoothfd_cell(Ycell,Tcell,Lfd_cell);


[newpars,newDEfd_cell] = Profile_GausNewt(theta',lsopts_out,DEfd,fn,...
    lambda,Ycell,Tcell,wts,[],lsopts_in);

disp(['New parameter estimates: ',num2str(newpars')]);

%%
clear
% First we need a function to calculate the functionl directly.

fn.fn       = @fhnfun;       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

fn.dfdx     = @fhndfdx;      % Derivative wrt inputs (Jacobian)
fn.dfdp     = @fhndfdp;      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

fn.d2fdx2   = @fhnd2fdx2;    % Hessian wrt inputs
fn.d2fdxdp  = @fhnd2fdxdp;   % Hessian wrt inputs and parameters
fn.d2fdp2   = @fhnd2fdp2;    % Hessian wrt parameters.



deb=0;
fin=20;
h_var = 0.1;

x0= [-1 1];
a = 0.2;
b = 0.2;
c = 2+1;
st_var = 0.2;

theta = [a b c];

  [T1 Y1] = fitz_nagumo(deb,fin,h_var,x0,a,b,c,st_var);
  
  Tcell = cell(1,2);
  Tcell{1} = T1;
  Tcell{2} = T1;
  
  Ycell = cell(1,2);
  Ycell{1} = Y1(:,1);
  Ycell{2} = Y1(:,2);

 [res_est res_x0] = est_param_gsa(Tcell,Ycell,theta,@fhnfun,@fhndfdx,@fhndfdp,@fhnd2fdx2,@fhnd2fdxdp,@fhnd2fdp2)

 %% lode model
clear 

% Variable initialisation
deb = 0;
fin = 5;
pas = (fin-deb)/50; 


param = 0.8;

x0 = 1;
std = 2;

%Data generation
 [res_t res_y ]= lineare_ode_sim(deb,pas,fin,param,x0,std);
 
  
  Tcell = cell(1,1);
  Tcell{1} = res_t(:,1);
  
  Ycell = cell(1,1);
  Ycell{1} = res_y(:,1);
  
  [res_est res_x0 newDEfd_cell] = est_param_gsa(Tcell,Ycell,param+2,@vf_lode_gsa,@dvf_dx_lode_gsa,@dvf_dp_lode_gsa,@d2vf_d2x_lode_gsa,@d2vf_dxdp_lode_gsa,@d2vf_d2p_lode_gsa)
  
  
%   newpreds = eval_fdcell(res_t,newDEfd_cell,0);
%   newpreds_der = eval_fdcell(res_t,newDEfd_cell,1);
%   
%   plot(res_t,newpreds{1},res_t,newpreds_der{1})

res_t = evaluate_controle_gsa(1,newDEfd_cell,@vf_lode_gsa,res_est)


 %% lode 2 param model
clear 

fn       = @vf_lode_2param_gsa;       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

dfdx     = @dvf_dx_lode_2param_gsa;      % Derivative wrt inputs (Jacobian)
dfdp     = @dvf_dp_lode_2param_gsa;      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

d2fdx2   = @d2vf_d2x_lode_2param_gsa;    % Hessian wrt inputs
d2fdxdp  = @d2vf_dxdp_lode_2param_gsa;   % Hessian wrt inputs and parameters
d2fdp2   = @d2vf_d2p_lode_2param_gsa;    % Hessian wrt parameters.

% Variable initialisation
deb = 0;
fin = 15;
pas = (fin-deb)/200;

param1 = 1.4;
param2 = 1;
param = [param1 param2];

x0 = 1;
std = 2;

%Data generation
 [res_t res_y ]= lineare_ode_sim_2param(deb,pas,fin,param,x0,std);
 
  
  Tcell = cell(1,1);
  Tcell{1} = res_t(:,1);
  
  Ycell = cell(1,1);
  Ycell{1} = res_y(:,1);
  
  [res_est res_x0 newDEfd_cell] = est_param_gsa(Tcell,Ycell,param+1,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2);
  
  
  newpreds = eval_fdcell(res_t,newDEfd_cell,0);
  newpreds_der = eval_fdcell(res_t,newDEfd_cell,1);
  
  plot(res_t,newpreds{1},res_t,newpreds_der{1})

res_t = evaluate_controle_gsa(1,newDEfd_cell,fn,res_est)

 %% A pinene model
clear 

fn       = @vf_apinene_gsa;       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

dfdx     = @dvf_dx_apinene_gsa;      % Derivative wrt inputs (Jacobian)
dfdp     = @dvf_dp_apinene_gsa;      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

d2fdx2   = @d2vf_d2x_apinene_gsa;    % Hessian wrt inputs
d2fdxdp  = @d2vf_dxdp_apinene_gsa;   % Hessian wrt inputs and parameters
d2fdp2   = @d2vf_d2p_apinene_gsa;    % Hessian wrt parameters.

% Variable initialisation
deb = 0;
fin = 15;
pas = (fin-deb)/200;

param1 = 1.4;
param2 = 1;
param = [param1 param2];

x0 = 1;
std = 2;

%Data generation
deb = 0;
fin = 100;
pas = (fin-deb)/100;
std = 5;

x_0_banga = [100 0 0 0 0];
theta_banga = 1000*[0.0000593 0.0000296 0.0000205 0.000275 0.00004];

%Data generation
[T1 Y1] = FBA_gen_sim(deb,fin,pas,[theta_banga x_0_banga],std);


  Tcell = cell(1,5);
  Tcell{1} = T1(:,1);
  Tcell{2} = T1(:,1);
  Tcell{3} = T1(:,1);
  Tcell{4} = T1(:,1);
  Tcell{5} = T1(:,1);
  
  Ycell = cell(1,5);
  Ycell{1} = Y1(:,1);
  Ycell{2} = Y1(:,2);
  Ycell{3} = Y1(:,3);
  Ycell{4} = Y1(:,4);
  Ycell{5} = Y1(:,5);
  
   [res_est res_x0 newDEfd_cell] = est_param_gsa(Tcell,Ycell,theta_banga,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2)
   [res_est_dc res_x0_dc newDEfd_cell_dc] = est_param_gsa(Tcell,Ycell,theta_banga+0.05,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2)
   
  newpreds = eval_fdcell(T1,newDEfd_cell,0);
  newpreds_der = eval_fdcell(T1,newDEfd_cell,1);
  
  plot(T1,newpreds{3},T1,newpreds_der{3})

res_t = evaluate_controle_gsa(1,newDEfd_cell,fn,res_est)
  

 %% Mat 2 param tot obs model
clear 

fn       = @vf_mat2param_partobs;       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

dfdx     = @dvf_dx_mat2param_partobs;      % Derivative wrt inputs (Jacobian)
dfdp     = @dvf_dp_mat2param_partobs;      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

d2fdx2   = @d2vf_d2x_mat2param_partobs;    % Hessian wrt inputs
d2fdxdp  = @d2vf_dxdp_mat2param_partobs;   % Hessian wrt inputs and parameters
d2fdp2   = @d2vf_d2p_mat2param_partobs;    % Hessian wrt parameters.


deb = 0;
fin = 100;
pas = (fin-deb)/100;
x_0 = [100 0 0];
theta_test = 1000*[0.0000593 0.0000296];
std = 2;

%Data generation
[T1 Y1] = Mat2param_sim(deb,fin,pas,theta_test,x_0,std)


  Tcell = cell(1,3);
  Tcell{1} = T1(:,1);
  Tcell{2} = T1(:,1);
  Tcell{3} = T1(:,1);

  
  Ycell = cell(1,3);
  Ycell{1} = Y1(:,1);
  Ycell{2} = Y1(:,2);
  Ycell{3} = Y1(:,3);

%   
   [res_est res_x0 newDEfd_cell] = est_param_gsa(Tcell,Ycell,theta_test,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2)
%    [res_est_dc res_x0_dc newDEfd_cell_dc] = est_param_gsa(Tcell,Ycell,theta_banga+0.05,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2)
%    
%   newpreds = eval_fdcell(T1,newDEfd_cell,0);
%   newpreds_der = eval_fdcell(T1,newDEfd_cell,1);
%   
%   plot(T1,newpreds{3},T1,newpreds_der{3})
% 
% res_t = evaluate_controle_gsa(1,newDEfd_cell,fn,res_est)
%   
% 
%   
%  

 %% Mat 2 param part obs model
clear 

fn       = @vf_mat2param_partobs;       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

dfdx     = @dvf_dx_mat2param_partobs;      % Derivative wrt inputs (Jacobian)
dfdp     = @dvf_dp_mat2param_partobs;      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

d2fdx2   = @d2vf_d2x_mat2param_partobs;    % Hessian wrt inputs
d2fdxdp  = @d2vf_dxdp_mat2param_partobs;   % Hessian wrt inputs and parameters
d2fdp2   = @d2vf_d2p_mat2param_partobs;    % Hessian wrt parameters.


deb = 0;
fin = 100;
pas = (fin-deb)/100;
x_0 = [100 0 0];
theta_test = 1000*[0.0000593 0.0000296];
std = 3;
partiel = 1;

%Data generation
[T1 Y1] = Mat2param_sim(deb,fin,pas,theta_test,x_0,std,partiel)


  Tcell = cell(1,3);
  Tcell{1} = [];
  Tcell{2} = T1(:,1);
  Tcell{3} = T1(:,1);

  
  Ycell = cell(1,3);
  Ycell{1} = [];
  Ycell{2} = Y1(:,1);
  Ycell{3} = Y1(:,2);

%   
%     [res_est res_x0 newDEfd_cell] = est_param_gsa(Tcell,Ycell,theta_test,fn, dfdx, dfdp, d2fdx2, d2fdxdp)
%    [res_est_dc res_x0_dc newDEfd_cell_dc] = est_param_gsa(Tcell,Ycell,theta_banga+0.05,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2)
%    
     newpreds = eval_fdcell(T1,newDEfd_cell,0);
%   newpreds_der = eval_fdcell(T1,newDEfd_cell,1);
%   
    plot(T1,newpreds{1})
% 
% res_t = evaluate_controle_gsa(1,newDEfd_cell,fn,res_est)
%   
 %% Methanation model
clear 


V = 0.04;
V_p = 0.05;
F_o_co = 0.45;
F_i_co = 0.59;
z_i_co = 0.132;
C_co = 0.02;
C_h2o = 0.01;
C_co2 = 0.01;
Weight = 0.744;
beta = 206.1;

theta_addi = [V V_p  F_o_co F_i_co z_i_co C_co C_h2o C_co2 Weight beta ];
coeff_mult = 20;


fn       = @(t,fd_cell,p)(vf_methanation(t,fd_cell,p,theta_addi,coeff_mult));       % RHS function

% Now derivatives of the function with respect to system components and
% parameters

dfdx     = @(t,fd_cell,p)(dvf_dx_methanation(t,fd_cell,p,theta_addi,coeff_mult));      % Derivative wrt inputs (Jacobian)
dfdp     = @(t,fd_cell,p)(dvf_dp_methanation(t,fd_cell,p,theta_addi,coeff_mult));      % Derviative wrt parameters

% Now we need functions to compute all three sets of second derivatives:

d2fdx2   = @(t,fd_cell,p)(d2vf_d2x_methanation(t,fd_cell,p,theta_addi,coeff_mult));    % Hessian wrt inputs
d2fdxdp  = @(t,fd_cell,p)(d2vf_dxdp_methanation(t,fd_cell,p,theta_addi,coeff_mult));   % Hessian wrt inputs and parameters


C_col = 5;
C_os = 15;
v5 = 0.4;
v6 = 0.5;
coeff_v6 = 0.015;

theta = [C_col C_os v5 v6];
x0 = [0 0 0 0]; 
 


std = 0.02;
deb = 0;
fin = 40;
h = 0.2;


[T1 Y1] = methanation_sim(deb,fin,h,theta_addi,theta,x0,std,@(t)(0),coeff_mult);


  Tcell = cell(1,4);
  Tcell{1} = T1(:,1);
  Tcell{2} = T1(:,1);
  Tcell{3} = T1(:,1);
  Tcell{4} = [];

   
  Ycell = cell(1,4);
  Ycell{1} = Y1(:,1);
  Ycell{2} = Y1(:,2);
  Ycell{3} = Y1(:,3);
  Ycell{4} = [];
 
     [res_est res_x0 newDEfd_cell] = est_param_gsa(Tcell,Ycell,0.8*theta,fn, dfdx, dfdp, d2fdx2, d2fdxdp)
%    [res_est_dc res_x0_dc newDEfd_cell_dc] = est_param_gsa(Tcell,Ycell,theta_banga+0.05,fn, dfdx, dfdp, d2fdx2, d2fdxdp, d2fdp2)
%    
%     newpreds = eval_fdcell(T1,newDEfd_cell,0);
%   newpreds_der = eval_fdcell(T1,newDEfd_cell,1);
%   
    plot(T1,Ycell{1},T1,Ycell{2},T1,Ycell{3})
% 
% res_t = evaluate_controle_gsa(1,newDEfd_cell,fn,res_est)
