%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%            Optimal control and additive perturbations help in estimating ill-posed and uncertain dynamical systems                    %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


                                                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                                        %%%% Bacterial Population Model       %%%
                                                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DATA GENERATION  & NON PARAMETRIC ESTIMATION IN WELL-SPECIFIED AND MIS-SPECIFIED CASE (BOTH STOCHASTIC AND DETERMINISTIC)

%parameter value
deb=0;
fin=8;
h_var = 0.08;
std = 2;

gamma = 2.4*10^-4;
rho = 10^-6;
psi = 0.7;
c = 0.1;

beta = 3;
Pmax = 1.8*10^5;
delta = 6*10^-6;
delta_p = 1.512;
nu_s = 24;
K = 10^9;
E_s =  36;
A = 4;
A_s50 = 0.25;

gamma_max = 0.035;
A_s = 4;
gamma_50 = 5;
lambda_bac = 10^-5;

A_r50 = 5;
E_r = 26.4;

theta = [gamma  rho psi c];
theta_log = [log(theta(1)) log(theta(2)) log(theta(3)) log(theta(4))];
theta_fixe = [beta Pmax delta delta_p nu_s K E_s A A_s50 gamma_max A_s gamma_50 lambda_bac  A_r50 E_r];

theta_sp = [gamma  psi c];
theta_log_sp = [log(theta(1))  log(theta(3)) log(theta(4))];
theta_fixe_sp = [beta Pmax delta delta_p nu_s K E_s A A_s50 gamma_max A_s gamma_50 lambda_bac  A_r50 E_r rho];

x0= [0 600 2];

 %Data in well-specified case
 [T1, Y1] = bacterial_fitness_sim(deb,h_var,fin,theta,theta_fixe,x0,std);
 Y_bact_wellspe = Y1(:,2) + Y1(:,3);
 
 
 %Data in the deterministic model misspecification case
 c_decr = 0.01;
 [T_detmisspe, Y_detmisspe] = bacterial_fitness_decr_eff_sim(deb,h_var,fin,theta,theta_fixe,x0,c_decr,std);
  Y_bact_detmisspe = Y_detmisspe(:,2) + Y_detmisspe(:,3);
  
  
  %Data in the stochastic disturbance case 
  sigma_dyn = 1;
  control_rand = @(t)([sigma_dyn*randn(1,1) sigma_dyn*randn(1,1) sigma_dyn*randn(1,1)]');
  [T_stomisspe, Y_stomisspe] = bacterial_fitness_sim(deb,h_var,fin,theta,theta_fixe,x0,std,control_rand);
  Y_bact_stomisspe = Y_stomisspe(:,2) + Y_stomisspe(:,3);
 
  
  %Non parametric estimator in well-specified case 
  Obs_l = Observation(T1,[Y1(:,1) Y_bact_wellspe]); 
  bs_l = bSplines(Obs_l);
  
  figure
  bs_l.erreur(1)
  title('bs l and data')
  
    
  %Non parametric estimator in the stochastic disturbance case 
  Obs_stomisspe = Observation(T_stomisspe,[Y_stomisspe(:,1) Y_bact_stomisspe]); 
  bs_stomisspe = bSplines(Obs_stomisspe);
  
  figure
  bs_stomisspe.erreur(1)
  title('bs stomisspe and data')
  
  
  %Non parametric estimator in the deterministic model misspecification case 
  Obs_detmisspe = Observation(T_detmisspe,[Y_detmisspe(:,1) Y_bact_detmisspe]); 
  bs_detmisspe = bSplines(Obs_detmisspe);
  
  figure
  bs_detmisspe.erreur(1)
  title('bs detmisspe and data')
  
  
  
 %% ESTIMATION IN THE WELL-SPECIFIED CASE
 
 % Required vector field and derivative
    v_field = @(t,vx,param,pf)vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field = @(t,vx,param,pf)der_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_theta = @(t,vx,param,pf)der_param_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_state_theta = @(t,vx,param,pf) der_state_param_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_state_state = @(t,vx,param,pf)der_sec_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    

   %Required matrix definitions
    mat_C_ext = zeros(2,3);
    mat_C_ext(1,1) = 1;
    mat_C_ext(2,2) = 1;
    mat_C_ext(2,3) = 1;
    
    intervalle_int2 = zeros(2,2);
    intervalle_int2(1,:) = [0 T1(50)];
    intervalle_int2(2,:) = [T1(51) T1(100)];

    lambda_trial =  [50 500 5000 50000 500000];

    %parameter initialization for the algorithm
    min_err_pred2 = 10^50;
    param_ini = theta_log_sp;
   
  warning off  
  for nl = 1 : length(lambda_trial)

        mat_U = lambda_trial(nl)*eye(3);
        
         %Parameter estimation for a given matrix U
        [est_param_oca,fval] = est_param_pontryagin_bvp4c_adjoint_method...
            (bs_l,v_field,der_v_field,der_v_field_theta,der_v_field_state_theta,der_v_field_state_state,mat_C_ext,mat_U,param_ini, x0);  
        param_ini = est_param_oca;
        
        %Control and forced solution reconstruction
        bs_u = est_control_pontryagin(bs_l,v_field,der_v_field,mat_C_ext,mat_U,est_param_oca,x0);

        %Error Prediction estimation
        res_err_pred2 = pontryagin_prediction_error_multi_inter(bs_l,bs_u,v_field,mat_C_ext,est_param_oca,x0,intervalle_int2);
   
      if res_err_pred2 < min_err_pred2
          min_err_pred2 = res_err_pred2;
          est_param_oc_err_pred2 = est_param_oca;
          lambda_err_pred2 = lambda_trial(nl);
      end
   
  end
   warning on
   
      
%Plot State Variables corresponding the solution of the optimal control problem
sol_Xu = pontryagin_compute_controlled_solution(bs_u,v_field,est_param_oc_err_pred2,x0);
sol_Xu_T = deval(sol_Xu,T1);
plot(T1,sol_Xu_T(1,:),'b',T1,sol_Xu_T(2,:),'g',T1,sol_Xu_T(3,:),'r')
   
%% ESTIMATION IN THE STOCHASTIC DISTURBANCE CASE
 
 % Required vector field and derivative
    v_field = @(t,vx,param,pf)vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field = @(t,vx,param,pf)der_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_theta = @(t,vx,param,pf)der_param_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_state_theta = @(t,vx,param,pf) der_state_param_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_state_state = @(t,vx,param,pf)der_sec_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    

   %Required matrix definitions
    mat_C_ext = zeros(2,3);
    mat_C_ext(1,1) = 1;
    mat_C_ext(2,2) = 1;
    mat_C_ext(2,3) = 1;
    
    intervalle_int2 = zeros(2,2);
    intervalle_int2(1,:) = [0 T1(50)];
    intervalle_int2(2,:) = [T1(51) T1(100)];
    lambda_trial =   [100 1000 10000 100000];

    %parameter initialization for the algorithm
    min_err_pred2 = 10^50;
    param_ini = theta_log_sp;
   
  warning off  
  for nl = 1 : length(lambda_trial)

        mat_U = lambda_trial(nl)*eye(3);
        
         %Parameter estimation for a given matrix U
        [est_param_oca,fval] = est_param_pontryagin_bvp4c_adjoint_method...
            (bs_stomisspe,v_field,der_v_field,der_v_field_theta,der_v_field_state_theta,der_v_field_state_state,mat_C_ext,mat_U,param_ini, x0);  
        param_ini = est_param_oca;
        
        %Control and forced solution reconstruction
        bs_u = est_control_pontryagin(bs_stomisspe,v_field,der_v_field,mat_C_ext,mat_U,est_param_oca,x0);

        %Error Prediction estimation
        res_err_pred2 = pontryagin_prediction_error_multi_inter(bs_stomisspe,bs_u,v_field,mat_C_ext,est_param_oca,x0,intervalle_int2);
   
      if res_err_pred2 < min_err_pred2
          min_err_pred2 = res_err_pred2;
          est_param_oc_err_pred2 = est_param_oca;
          lambda_err_pred2 = lambda_trial(nl);
      end
   
  end
   warning on
    
   
%Plot State Variables corresponding the solution of the optimal control problem
sol_Xu = pontryagin_compute_controlled_solution(bs_u,v_field,est_param_oc_err_pred2,x0);
sol_Xu_T = deval(sol_Xu,T_stomisspe);
plot(T_detmisspe,sol_Xu_T(1,:),'b',T_stomisspe,sol_Xu_T(2,:),'g',T_stomisspe,sol_Xu_T(3,:),'r')
   
%% ESTIMATION IN THE DETERMINISTIC MIS-SPECIFIED CASE
 
 % Required vector field and derivative
    v_field = @(t,vx,param,pf)vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field = @(t,vx,param,pf)der_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_theta = @(t,vx,param,pf)der_param_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_state_theta = @(t,vx,param,pf) der_state_param_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    der_v_field_state_state = @(t,vx,param,pf)der_sec_vect_field_bact_fitness_exp_sp(t,vx,param,theta_fixe_sp);
    

   %Required matrix definitions
    mat_C_ext = zeros(2,3);
    mat_C_ext(1,1) = 1;
    mat_C_ext(2,2) = 1;
    mat_C_ext(2,3) = 1;
    
    intervalle_int2 = zeros(2,2);
    intervalle_int2(1,:) = [0 T1(50)];
    intervalle_int2(2,:) = [T1(51) T1(100)];

    lambda_trial =   [500 1000 5000 10000];

    %parameter initialization for the algorithm
    min_err_pred2 = 10^50;
    param_ini = theta_log_sp;
   
  warning off  
  for nl = 1 : length(lambda_trial)

        mat_U = lambda_trial(nl)*eye(3);
        
         %Parameter estimation for a given matrix U
        [est_param_oca,fval] = est_param_pontryagin_bvp4c_adjoint_method...
            (bs_detmisspe,v_field,der_v_field,der_v_field_theta,der_v_field_state_theta,der_v_field_state_state,mat_C_ext,mat_U,param_ini, x0);  
        param_ini = est_param_oca;
        
        %Control and forced solution reconstruction
        bs_u = est_control_pontryagin(bs_detmisspe,v_field,der_v_field,mat_C_ext,mat_U,est_param_oca,x0);

        %Error Prediction estimation
        res_err_pred2 = pontryagin_prediction_error_multi_inter(bs_detmisspe,bs_u,v_field,mat_C_ext,est_param_oca,x0,intervalle_int2);
   
      if res_err_pred2 < min_err_pred2
          min_err_pred2 = res_err_pred2;
          est_param_oc_err_pred2 = est_param_oca;
          lambda_err_pred2 = lambda_trial(nl);
      end
   
  end
   warning on
    
   
%Plot State Variables corresponding the solution of the optimal control problem
sol_Xu = pontryagin_compute_controlled_solution(bs_u,v_field,est_param_oc_err_pred2,x0);
sol_Xu_T = deval(sol_Xu,T_detmisspe);
plot(T_detmisspe,sol_Xu_T(1,:),'b',T_detmisspe,sol_Xu_T(2,:),'g',T_detmisspe,sol_Xu_T(3,:),'r')


%% 

                                                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                                        %%%%        FitzHugh-Nagumo          %%%%
                                                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DATA GENERATION  & NON PARAMETRIC ESTIMATION IN WELL-SPECIFIED AND MIS-SPECIFIED CASE 

%Parameter values
a = 0.2;
b = 0.2;
c = 3;
theta = [a b c];

x_0= [-1 1];


deb = 0;
fin = 20;
pas = 0.2;
std = 0.1;

step_fun = @(t)([c*0.15*step_function(t) 0]');


%data with step-function
[T1 Y1] = fitz_nagumo(deb,fin,pas,x_0,a,b,c,std,step_fun);

%Non parametric estimator
Obs_l = Observation(T1,Y1); 
bs_l = bSplines(Obs_l);
  
figure
bs_l.erreur(1)
title('bs l and data')

 %% ESTIMATION 
 
 % Required vector field and derivative
    v_field = @vect_field_fitzHugh;
    der_v_field = @der_vect_field_fitzHugh;
    der_v_field_theta = @der_param_vect_field_fitzHugh;
    der_v_field_state_theta = @der_state_param_vect_field_fitzHugh;
    der_v_field_state_state = @der_sec_vect_field_fitzHugh;
    

   %Required matrix definitions
    mat_C_ext = eye(2);
    
    intervalle_int2 = zeros(2,2);
    intervalle_int2(1,:) = [0 T1(50)];
    intervalle_int2(2,:) = [T1(51) T1(100)];

    lambda_trial =   [5 10 20 50];

    %parameter initialization for the algorithm
    min_err_pred2 = 10^50;
    param_ini = theta;
   
  warning off  
  for nl = 1 : length(lambda_trial)

   try   
        mat_U = lambda_trial(nl)*eye(2);
        
         %Parameter estimation for a given matrix U
        [est_param_oca,fval] = est_param_pontryagin_bvp4c_adjoint_method...
            (bs_l,v_field,der_v_field,der_v_field_theta,der_v_field_state_theta,der_v_field_state_state,mat_C_ext,mat_U,param_ini,x_0);  
        param_ini = est_param_oca;
        
        %Control and forced solution reconstruction
        bs_u = est_control_pontryagin(bs_l,v_field,der_v_field,mat_C_ext,mat_U,est_param_oca,x_0);

        %Error Prediction estimation
        res_err_pred2 = pontryagin_prediction_error_multi_inter(bs_l,bs_u,v_field,mat_C_ext,est_param_oca,x_0,intervalle_int2);
   
      if res_err_pred2 < min_err_pred2
          min_err_pred2 = res_err_pred2;
          est_param_oc_err_pred2 = est_param_oca;
          lambda_err_pred2 = lambda_trial(nl);
      end
   catch
         'BVP integration error'
         nl
     end
  end
   warning on
   
      
%Plot State Variables corresponding the solution of the optimal control problem
sol_Xu = pontryagin_compute_controlled_solution(bs_u,v_field,est_param_oc_err_pred2,x_0);
sol_Xu_T = deval(sol_Xu,T1);
plot(T1,sol_Xu_T(1,:),'b',T1,sol_Xu_T(2,:),'g')

%%
 

                                                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                                        %%%% Cells growth and migration model  %%
                                                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DATA GENERATION  & NON PARAMETRIC ESTIMATION IN WELL-SPECIFIED AND MIS-SPECIFIED CASE 


% Non parametric estimation of Dm
load('ensemble_donne_utile.mat')
Obs_dend = Observation(data_moy(:,1),data_moy(:,2));
bs_dend = bSplines(Obs_dend);
func_inter_dend  = @(t)(bs_dend.calcul_prediction(t,0));

%Plot DM
sample_dend = bs_dend.calcul_prediction(2:0.1:15);
sample_dend = arrayfun(@(x)10^x,sample_dend);
figure
plot(2:0.1:15,sample_dend)
title('Dm')

%Parameter values
rho_m = 1.58*10^-5;
rho_s = 3.59*10^-5;
sigma_m_s = 1.77E-1;
sigma_s_l = 3.52E-1;
Tau = 3.53E+0;

%functional parameters
gamma_l_function = @(t)(t+sin(pi*t/9));
nu_s = @(t)((t.^2)/10);

%Plot functional parameters
gamma_l_t = arrayfun(gamma_l_function,0:0.1:9);
figure
plot(0:0.1:9,gamma_l_t)
title('delta_l')

nu_s_t = arrayfun(nu_s,0:0.1:9);
figure
plot(0:0.1:9,nu_s_t)
title('nu_s')

%Observation interval & measurement noise setting
deb_sf = 0;
fin_sf = 9;
pas = 100;
sf = 5;
std = 0.2;

vect_param = [rho_m rho_s  sigma_m_s sigma_s_l];
vect_CI = [log(4430) log(3490) log(1330)];
    
  
 %Data in well-specified case
 [Y1, T1] = log_influenza_sim(deb_sf,pas,fin_sf,vect_param,vect_CI,bs_dend,Tau,std,sf,gamma_l_function);
 
 %Data in mis-specified case
 [Y1_nus, T1_nus] = log_influenza_nus_sim(deb_sf,pas,fin_sf,vect_param,vect_CI,bs_dend,Tau,std,sf,gamma_l_function,nu_s);

 
  %Non parametric estimator in well-specified case 
  Obs_l = Observation(T1,Y1); 
  bs_l = bSplines(Obs_l);
  
  figure
  bs_l.erreur(1)
  title('bs l and data')

  
  %Non parametric estimator in mis-specified case 
  Obs_nus = Observation(T1_nus,Y1_nus); 
  bs_nus = bSplines(Obs_nus);    
  
  figure
  bs_nus.erreur(1)
  title('bs nus and data')
    
%% ESTIMATION IN THE WELL-SPECIFIED CASE
    
% Required vector field and derivative
    v_field = @(t,vx,param,param_fixe)vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field = @(t,vx,param,param_fixe)der_vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field_theta = @(t,vx,param,param_fixe)der_param_vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field_state_theta = @(t,vx,param,param_fixe)der_state_param_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field_state_state = @(t,vx,param,param_fixe)der_sec_vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    


%Required matrix definitions
mat_C_ext = zeros(3,5);
mat_C_ext(1:3,1:3) = eye(3);

mat_B_ext = zeros(5,4);
mat_B_ext(1:3,1:3) = eye(3);
mat_B_ext(5,4) = 1;

lambda_trial = [100 1000 10000 100000];

    
    intervalle_int2 = zeros(2,2);
    intervalle_int2(1,:) = [T1(1) T1(50)];
    intervalle_int2(2,:) = [T1(50) T1(100)];
    
    
    %Required parameter initialisation for the algoritm
    err_pred2_cur = 10^50; 
    log_vect_param = [log(rho_m) log(rho_s)  log(sigma_m_s) log(sigma_s_l)];
    param_ini_oca = 0.9*[log_vect_param 0 (1+pi/9)];
    
      for nl = 1 : length(lambda_trial)

          mat_U = zeros(4);
          mat_U(1:3,1:3) = lambda_trial(nl)*eye(3);
          mat_U(4,4) = 0.1;

        warning off
        try
         %Parameter estimation for a given matrix U
        [est_param_oca_tot_cur,fval_tr_dc] = est_param_pontryagin_bvp4c_adjoint_method...
            (bs_l,v_field,der_v_field,der_v_field_theta,der_v_field_state_theta,der_v_field_state_state,mat_C_ext,mat_U,param_ini_oca,vect_CI,mat_B_ext);  

         param_ini_oca = est_param_oca_tot_cur;
         
         est_param_oca_cur =  est_param_oca_tot_cur(1:4);
         est_vect_CI_ext = [vect_CI est_param_oca_tot_cur(5:6)];

         %Control and forced solution reconstruction
         bs_u = est_control_pontryagin(bs_l,v_field,der_v_field,mat_C_ext,mat_U,est_param_oca_cur,est_vect_CI_ext,mat_B_ext);
         sol_Xu = pontryagin_compute_controlled_solution(bs_u,v_field,est_param_oca_cur,est_vect_CI_ext,mat_B_ext);
         sol_Xu_T1 = deval(sol_Xu,T1);

         %Functional parameter estimation
         bs_gl = bSplines(Observation(T1,sol_Xu_T1(4,:)'));
         v_field_gl = @(t,vx,param)vect_field_influenza_log_gl(t,vx,param,[Tau sf],@(t)(10^func_inter_dend(t)),bs_gl);

         %Error Prediction estimation
         res_err_pred2 = pontryagin_prediction_error_semiparam_multi_inter...
             (bs_l,bs_u,v_field,v_field_gl,eye(3),est_param_oca_cur,est_vect_CI_ext,intervalle_int2,mat_B_ext);
         
         if res_err_pred2 < err_pred2_cur
             err_pred2_cur = res_err_pred2;
             est_param_oca_err_pred2 = est_param_oca_cur;
             bs_gl_oca_err_pred2 = bs_gl;     
         end

        catch     
            'echec' 
            nl
        end
        warning on

      end 

         
     %Plot functional parameter estimation  
      figure
      bs_gl_T1 =  bs_gl_oca_err_pred2.calcul_prediction(T1,0);
      plot(T1, bs_gl_T1)
      title('Estimation delta_l wellpe')
      
%% ESTIMATION IN THE MIS-SPECIFIED CASE
    
% Required vector field and derivative
    v_field = @(t,vx,param,param_fixe)vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field = @(t,vx,param,param_fixe)der_vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field_theta = @(t,vx,param,param_fixe)der_param_vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field_state_theta = @(t,vx,param,param_fixe)der_state_param_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    der_v_field_state_state = @(t,vx,param,param_fixe)der_sec_vect_field_influenza_fun_log(t,vx,param,param_fixe,[Tau sf],@(t)(10^func_inter_dend(t)));
    


%Required matrix definitions
    mat_C_ext = zeros(3,5);
    mat_C_ext(1:3,1:3) = eye(3);

    mat_B_ext = zeros(5,4);
    mat_B_ext(1:3,1:3) = eye(3);
    mat_B_ext(5,4) = 1;

    lambda_trial = [100 1000 10000 100000];

    
    intervalle_int2 = zeros(2,2);
    intervalle_int2(1,:) = [T1_nus(1) T1_nus(50)];
    intervalle_int2(2,:) = [T1_nus(50) T1_nus(100)];
    
    
    %Required parameter initialisation for the algoritm
    err_pred2_cur = 10^50; 
    log_vect_param = [log(rho_m) log(rho_s)  log(sigma_m_s) log(sigma_s_l)];
    param_ini_oca = 0.9*[log_vect_param 0 (1+pi/9)];
    
      for nl = 1 : length(lambda_trial)

          mat_U = zeros(4);
          mat_U(1:3,1:3) = lambda_trial(nl)*eye(3);
          mat_U(4,4) = 0.1;

        warning off
        try
            
        %Parameter estimation for a given matrix U    
        [est_param_oca_tot_cur,fval_tr_dc] = est_param_pontryagin_bvp4c_adjoint_method...
            (bs_nus,v_field,der_v_field,der_v_field_theta,der_v_field_state_theta,der_v_field_state_state,mat_C_ext,mat_U,param_ini_oca,vect_CI,mat_B_ext);  

         param_ini_oca = est_param_oca_tot_cur;
         
         est_param_oca_cur =  est_param_oca_tot_cur(1:4);
         est_vect_CI_ext = [vect_CI est_param_oca_tot_cur(5:6)];

         %Control and forced solution reconstruction
         bs_u = est_control_pontryagin(bs_nus,v_field,der_v_field,mat_C_ext,mat_U,est_param_oca_cur,est_vect_CI_ext,mat_B_ext);
         sol_Xu = pontryagin_compute_controlled_solution(bs_nus,v_field,est_param_oca_cur,est_vect_CI_ext,mat_B_ext);
         sol_Xu_T1_nus = deval(sol_Xu,T1_nus);

         %Functional parameter estimation
         bs_gl = bSplines(Observation(T1_nus,sol_Xu_T1_nus(4,:)'));
         v_field_gl = @(t,vx,param)vect_field_influenza_log_gl(t,vx,param,[Tau sf],@(t)(10^func_inter_dend(t)),bs_gl);

         %Error Prediction estimation
         res_err_pred2 = pontryagin_prediction_error_semiparam_multi_inter...
             (bs_nus,bs_u,v_field,v_field_gl,eye(3),est_param_oca_cur,est_vect_CI_ext,intervalle_int2,mat_B_ext)
         
         if res_err_pred2 < err_pred2_cur
             err_pred2_cur = res_err_pred2;
             est_param_oca_err_pred2 = est_param_oca_cur;
             bs_gl_oca_err_pred2 = bs_gl;      
         end

        catch     
            'echec' 
            nl
        end
        warning on

      end 

      
     %Plot functional parameter estimation  
      figure
      bs_gl_T1_nus =  bs_gl_oca_err_pred2.calcul_prediction(T1_nus,0);
      plot(T1_nus, bs_gl_T1_nus)
      title('Estimation delta_l misspe')
      